"use strict";
/**
 * This module is used to make the BufferUI interact with an ArcGIS API for JavaScript v3 map.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var arcgis_buffer_ui_1 = require("@wsdot/arcgis-buffer-ui");
var PopupTemplate = require("esri/PopupTemplate");
var geometryEngineAsync = require("esri/geometry/geometryEngineAsync");
var geometryJsonUtils = require("esri/geometry/support/jsonUtils");
var Graphic = require("esri/Graphic");
var FeatureLayer = require("esri/layers/FeatureLayer");
// /**
//  * Adds a "buffer" link to an InfoWindow's ".actionList" section.
//  * When clicked it will add the selected feature
//  * to the BufferUI geometries list.
//  * @param infoWindow - A Popup
//  * @param bufferUI - A BufferUI object.
//  */
// export function addBufferLink(infoWindow: Popup, bufferUI: BufferUI) {
//   const action: any = {
//     title: "Buffer current geometry",
//     id: "buffer",
//     className: "action-buffer"
//   };
//   infoWindow.actions.push(action);
//   infoWindow.on("trigger-action", (event) => {
//     bufferUI.addFeature(infoWindow.selectedFeature);
//   })
// }
/**
 * Creates a feature layer and adds it to the map.
 * @param view
 * @param buffer
 * @param layerId - Specifies the id property's value of the layer FeatureLayer that will be used to display the buffer polygons.
 * @returns Returns the FeatureLayer that was created to display buffer polygons.
 */
function attachBufferUIToMap(view, buffer, layerId) {
    var _this = this;
    if (layerId === void 0) { layerId = "Buffer"; }
    var oid = 0;
    var popupTemplate = new PopupTemplate({
        title: "Buffer",
        actions: [
            {
                title: "Buffer current geometry",
                id: "buffer",
                className: "action-buffer"
            }
        ],
        fieldInfos: [
            {
                fieldName: "distance",
                label: "Buffer Distance",
                visible: true,
                format: {
                    places: 0,
                    digitSeparator: true
                }
            },
            {
                fieldName: "unit",
                label: "Measurement Unit",
                visible: true
            },
            {
                fieldName: "area",
                label: "Area",
                visible: true,
                format: {
                    places: 0,
                    digitSeparator: true
                }
            },
            {
                fieldName: "areaUnit",
                label: "Area Unit",
                visible: true
            }
        ]
    });
    var bufferFeatureLayer = new FeatureLayer({
        geometryType: "esriGeometryPolygon",
        fields: [
            {
                name: "oid",
                type: "esriFieldTypeOID"
            },
            {
                name: "distance",
                type: "esriFieldTypeDouble"
            },
            {
                name: "unit",
                type: "esriFieldTypeString",
                alias: "Measurement Unit"
            },
            {
                name: "unioned",
                type: "esriFieldTypeSmallInteger",
                alias: "Is Unioned"
            },
            {
                name: "area",
                type: "esriFieldTypeDouble",
                alias: "Area"
            },
            {
                name: "areaUnit",
                type: "esriFieldTypeString",
                alias: "Area Unit"
            }
        ]
    });
    bufferFeatureLayer.popupTemplate = popupTemplate;
    view.map.add(bufferFeatureLayer);
    view.popup.on("trigger-action", function (event) {
        buffer.addFeature(view.popup.selectedFeature);
    });
    buffer.form.addEventListener("clear-graphics", function () { return __awaiter(_this, void 0, void 0, function () {
        var features, editResults;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, bufferFeatureLayer.queryFeatures()];
                case 1:
                    features = _a.sent();
                    return [4 /*yield*/, bufferFeatureLayer.applyEdits({
                            deleteFeatures: features.features
                        })];
                case 2:
                    editResults = _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    buffer.form.addEventListener("buffer", function (e) {
        var detail = e.detail;
        // Convert regular objects into esri/Geometry objects.
        if (Array.isArray(detail.geometry)) {
            detail.geometry = detail.geometry.map(geometryJsonUtils.fromJSON, detail.geometry);
        }
        else {
            detail.geometry = geometryJsonUtils.fromJSON(detail.geometry);
        }
        // The geometry engine requires that the number of geometries and distances be the same.
        // If multiple distances are provided but only a single geometry, that geometry will be
        // buffered for each distance.
        if (Array.isArray(detail.distance) && !Array.isArray(detail.geometry)) {
            detail.geometry = (function () {
                var outGeoArray = new Array(detail.distance.length);
                for (var i = 0, l = detail.distance.length; i < l; i += 1) {
                    outGeoArray[i] = detail.geometry;
                }
                return outGeoArray;
            })();
        }
        else if (!Array.isArray(detail.distance) &&
            Array.isArray(detail.geometry)) {
            detail.distance = (function () {
                var outDistanceArray = new Array();
                for (var i = 0; i < detail.geometry.length; i++) {
                    outDistanceArray[i] = detail.distance;
                }
                return outDistanceArray;
            })();
        }
        geometryEngineAsync
            .buffer(detail.geometry, detail.distance, detail.unit, detail.unionResults)
            .then(function (bufferResults) {
            console.log("buffer results", bufferResults);
            var unit = arcgis_buffer_ui_1.getUnitForId(detail.unit);
            // unit = unit.description;
            if (bufferResults) {
                if (!Array.isArray(bufferResults)) {
                    bufferResults = [bufferResults];
                }
                bufferResults.forEach(function (geometry, i) {
                    var promise = geometryEngineAsync
                        .planarArea(geometry, undefined)
                        .then(function (area) {
                        console.debug("area", area);
                        var acres = area / 4047;
                        var graphic = new Graphic({
                            geometry: geometry,
                            attributes: {
                                oid: oid++,
                                distance: Array.isArray(detail.distance)
                                    ? detail.distance[i]
                                    : detail.distance,
                                unit: unit.description,
                                unioned: detail.unionResults,
                                area: acres < 1 ? acres * 43560 : acres,
                                areaUnit: acres < 1 ? "ft\u00b2" : "ac" // ft. squared or acres
                            }
                        });
                        bufferFeatureLayer.applyEdits([graphic]);
                    }, function (error) {
                        console.error("area", error);
                        var graphic = new Graphic({
                            geometry: geometry,
                            attributes: {
                                oid: oid++,
                                distance: Array.isArray(detail.distance)
                                    ? detail.distance[i]
                                    : detail.distance,
                                unit: unit.description,
                                unioned: detail.unionResults,
                                areaError: error
                            }
                        });
                        bufferFeatureLayer.applyEdits([graphic]);
                    });
                });
            }
            buffer.clearGeometryList();
        }, function (error) {
            console.error("buffer error", error);
        });
    });
    return bufferFeatureLayer;
}
exports.attachBufferUIToMap = attachBufferUIToMap;
